package meerkat.mixer.proofs;

import com.google.protobuf.Message;

import java.math.BigInteger;

/**
 * Generic Sigma Protocol.
 * The challenge is always a {@link java.math.BigInteger}.
 */
public interface SigmaProtocol {
    public interface Prover <FirstMsgType, FinalMessageType> {
        public FirstMsgType getFirstMessage();
        public FinalMessageType getFinalMessage(BigInteger challenge);

        /**
         * Reset the prover (so the next calls to {@link #getFirstMessage()} will
         * start a new proof.
         */
        public void reset();
    }

    public interface Simulator <FirstMsgType, FinalMessageType> {
        public FirstMsgType getFirstMessage(BigInteger challenge);
        public FinalMessageType getFinalMessage();

        /**
         * Reset the prover (so the next calls to {@link #getFirstMessage(BigInteger)} will
         * start a new proof.
         */
        public void reset();
    }

    public interface Verifier <FirstMsgType, FinalMessageType>  {
        public boolean verify(FirstMsgType firstMessage, BigInteger challenge, FinalMessageType finalMessage);
    }
}
