package meerkat.mixer.proofs;

import meerkat.protobuf.Mixing;
import org.factcenter.qilin.util.Pair;

import java.math.BigInteger;

/**
 * Created by talm on 14/01/17.
 */
public class DlogOrStatementSigmaTest extends SigmaProtocolTest<Mixing.Mix2Proof.FirstMessage, Mixing.Mix2Proof.FinalMessage> {
    final DlogStatementSchnorrSigmaTest dlogtest;

    final ECElGamalMixParams.AndStatement[] statements = new ECElGamalMixParams.AndStatement[2];
    ECElGamalMixParams.AndStatementWitness w;
    int trueStatementIndex;


    public DlogOrStatementSigmaTest() {
        this.dlogtest = new DlogStatementSchnorrSigmaTest();
    }

    @Override
    void generateRandomTrueStatement() {
        trueStatementIndex = rand.nextInt(2);
        Pair<ECElGamalMixParams.DlogStatement, ECElGamalMixParams.DlogStatementWitness> s1w1 = dlogtest.returnRandomTrueStatement();
        Pair<ECElGamalMixParams.DlogStatement, ECElGamalMixParams.DlogStatementWitness> s2w2 = dlogtest.returnRandomTrueStatement();
        ECElGamalMixParams.AndStatement trueStatement = dlogtest.params.new AndStatement(s1w1.a, s2w2.a);
        w = dlogtest.params.new AndStatementWitness(s1w1.b, s2w2.b);
        statements[trueStatementIndex] = trueStatement;

        ECElGamalMixParams.DlogStatement f1 = dlogtest.returnRandomFalseStatement();
        ECElGamalMixParams.DlogStatement f2 = dlogtest.returnRandomFalseStatement();

        ECElGamalMixParams.AndStatement falseStatement = dlogtest.params.new AndStatement(f1, f2);

        statements[1 - trueStatementIndex] = falseStatement;
    }

    @Override
    void generateRandomFalseStatement() {
        ECElGamalMixParams.DlogStatement f1 = dlogtest.returnRandomFalseStatement();
        ECElGamalMixParams.DlogStatement f2 = dlogtest.returnRandomFalseStatement();

        statements[0] = dlogtest.params.new AndStatement(f1, f2);

        f1 = dlogtest.returnRandomFalseStatement();
        f2 = dlogtest.returnRandomFalseStatement();
        statements[1] = dlogtest.params.new AndStatement(f1, f2);
    }

    @Override
    protected SigmaProtocol.Prover<Mixing.Mix2Proof.FirstMessage, Mixing.Mix2Proof.FinalMessage> getNewProver() {
        SigmaProtocol.Prover<Mixing.Mix2Proof.AndProof.FirstMessage, Mixing.Mix2Proof.AndProof.FinalMessage> andProver = dlogtest.prots.new AndStatementProver(statements[trueStatementIndex], w);
        SigmaProtocol.Simulator<Mixing.Mix2Proof.AndProof.FirstMessage, Mixing.Mix2Proof.AndProof.FinalMessage> andSimulator = dlogtest.prots.new AndStatementSimulator(statements[1 - trueStatementIndex]);

        return new SigmaProtocolOr2.Prover<>(ProtobufConcatenators.concatMix1, ProtobufConcatenators.concatMix2,
                dlogtest.prots.challengeGenerator, andProver, andSimulator, trueStatementIndex);
    }

    @Override
    protected SigmaProtocol.Verifier<Mixing.Mix2Proof.FirstMessage, Mixing.Mix2Proof.FinalMessage>  getNewVerifier() {
        return new SigmaProtocolOr2.Verifier<Mixing.Mix2Proof.FirstMessage, Mixing.Mix2Proof.AndProof.FirstMessage,
                Mixing.Mix2Proof.FinalMessage, Mixing.Mix2Proof.AndProof.FinalMessage>(ProtobufConcatenators.concatMix1, ProtobufConcatenators.concatMix2,
                dlogtest.prots.challengeGenerator, dlogtest.prots.new AndStatementVerifier(statements[0]), dlogtest.prots.new AndStatementVerifier(statements[1]));
    }

    @Override
    protected SigmaProtocol.Simulator<Mixing.Mix2Proof.FirstMessage, Mixing.Mix2Proof.FinalMessage>  getNewSimulator() {
        return new SigmaProtocolOr2.Simulator<>(ProtobufConcatenators.concatMix1, ProtobufConcatenators.concatMix2,
                dlogtest.prots.challengeGenerator, dlogtest.prots.new AndStatementSimulator(statements[0]),
                dlogtest.prots.new AndStatementSimulator(statements[1]));
    }

    @Override
    protected BigInteger getChallengeModulus() {
        return dlogtest.getChallengeModulus();
    }
}
