package meerkat.mixer.proofs;

import meerkat.mixer.ECParamTestBase;
import meerkat.protobuf.Mixing;
import org.bouncycastle.math.ec.ECPoint;
import org.factcenter.qilin.util.Pair;

import java.math.BigInteger;

/**
 * Created by talm on 12/01/17.
 */
public class DlogStatementSchnorrSigmaTest extends
        SigmaProtocolTest<Mixing.Mix2Proof.DlogProof.FirstMessage, Mixing.Mix2Proof.DlogProof.FinalMessage> {

    ECParamTestBase ecParams = new ECParamTestBase();
    ECElGamalMixParams params;
    ECElGamalMixProtocols prots;

    ECElGamalMixParams.DlogStatement statement;
    ECElGamalMixParams.DlogStatementWitness witness;
    ECElGamalMixProtocols.DlogStatementSchnorrProver prover;
    ECElGamalMixProtocols.DlogStatementSchnorrVerifier verifier;
    ECElGamalMixProtocols.DlogStatementSchnorrSimulator simulator;

    public DlogStatementSchnorrSigmaTest() {

        this.params = new ECElGamalMixParams(ecParams.enc);
        this.prots = new ECElGamalMixProtocols(params, rand); // We don't need randomness for the verifier
    }


    Pair<ECElGamalMixParams.DlogStatement, ECElGamalMixParams.DlogStatementWitness> returnRandomTrueStatement() {
        BigInteger x = prots.encryptor.generateRandomExponent(rand);
        ECPoint a = prots.group.multiply(prots.g, x);
        ECPoint b = prots.group.multiply(prots.h, x);
        ECElGamalMixParams.DlogStatement statement = params.new DlogStatement(a, b);
        ECElGamalMixParams.DlogStatementWitness witness = params.new DlogStatementWitness(x);

        return new Pair<>(statement, witness);
    }

    ECElGamalMixParams.DlogStatement returnRandomFalseStatement() {
        ECPoint a = prots.group.sample(rand);
        ECPoint b = prots.group.sample(rand);

        return params.new DlogStatement(a, b);
    }

    void generateRandomTrueStatement() {
        Pair<ECElGamalMixParams.DlogStatement, ECElGamalMixParams.DlogStatementWitness> sw = returnRandomTrueStatement();
        this.statement = sw.a;
        this.witness = sw.b;
    }

    void generateRandomFalseStatement() {
        witness = null;
        statement = returnRandomFalseStatement();
    }

    @Override
    protected SigmaProtocol.Prover<Mixing.Mix2Proof.DlogProof.FirstMessage, Mixing.Mix2Proof.DlogProof.FinalMessage> getNewProver() {
        prover = prots.new DlogStatementSchnorrProver(statement, witness);
        return prover;
    }

    @Override
    protected SigmaProtocol.Verifier<Mixing.Mix2Proof.DlogProof.FirstMessage, Mixing.Mix2Proof.DlogProof.FinalMessage> getNewVerifier() {
        verifier = prots.new DlogStatementSchnorrVerifier(statement);
        return verifier;
    }

    @Override
    protected SigmaProtocol.Simulator<Mixing.Mix2Proof.DlogProof.FirstMessage, Mixing.Mix2Proof.DlogProof.FinalMessage> getNewSimulator() {
        simulator = prots.new DlogStatementSchnorrSimulator(statement);
        return simulator;
    }

    @Override
    protected BigInteger getChallengeModulus() {
        return prots.group.orderUpperBound();
    }
}